package de.fitko.fitconnect.core;

import com.google.inject.Inject;
import com.nimbusds.jose.jwk.RSAKey;
import de.fitko.fitconnect.api.domain.auth.OAuthToken;
import de.fitko.fitconnect.api.domain.model.metadata.Metadata;
import de.fitko.fitconnect.api.domain.model.metadata.attachment.Attachment;
import de.fitko.fitconnect.api.domain.model.metadata.attachment.signature.Hash__1;
import de.fitko.fitconnect.api.domain.model.metadata.attachment.signature.Type;
import de.fitko.fitconnect.api.domain.model.metadata.data.Data;
import de.fitko.fitconnect.api.domain.model.submission.SubmissionForPickup;
import de.fitko.fitconnect.api.domain.model.submission.SubmissionSubmit;
import de.fitko.fitconnect.api.domain.validation.ValidationResult;
import de.fitko.fitconnect.api.exceptions.AttachmentCreationException;
import de.fitko.fitconnect.api.services.Sender;
import de.fitko.fitconnect.api.services.auth.OAuthService;
import de.fitko.fitconnect.api.services.crypto.CryptoService;
import de.fitko.fitconnect.api.services.metadata.MetadataService;
import de.fitko.fitconnect.api.services.validation.CertificateValidator;
import de.fitko.fitconnect.core.crypto.HashUtil;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import java.io.File;
import java.io.IOException;
import java.util.List;
import java.util.UUID;

public class SubmissionSender implements Sender {

    private static final Logger logger = LoggerFactory.getLogger(SubmissionSender.class);

    private final OAuthService authService;
    private final CertificateValidator certificateValidator;
    private final CryptoService cryptoService;
    private final MetadataService metadataService;
    private final HashUtil hashUtil = new HashUtil();

    @Inject
    public SubmissionSender(final OAuthService authService,
                            final CryptoService encryptionService,
                            final CertificateValidator certificateValidator,
                            final MetadataService metadataService) {
        this.authService = authService;
        this.cryptoService = encryptionService;
        this.certificateValidator = certificateValidator;
        this.metadataService = metadataService;
    }


    @Override
    public ValidationResult validatePublicKey(final RSAKey publicKey) {
        return certificateValidator.validatePublicKey(publicKey);
    }

    @Override
    public Data encryptSubmissionData(final RSAKey publicKey, final Data data) {
        throw new UnsupportedOperationException("not yet implemented");
    }

    @Override
    public Attachment encryptAttachment(final RSAKey publicKey, final Attachment attachment) {
        throw new UnsupportedOperationException("not yet implemented");
    }

    @Override
    public OAuthToken retrieveOAuthToken(final String clientId, final String clientSecret, final String... scope) {
        final OAuthToken token = authService.authenticate(clientId, clientSecret, scope);
        logger.debug("Successfully retrieved OAuth token: {}", token.getAccessToken());
        return token;
    }

    @Override
    public Metadata createMetadata(final Data data, final List<Attachment> attachments) {
        throw new UnsupportedOperationException("not yet implemented");
    }

    @Override
    public Metadata createMetadata(final List<Attachment> attachments) {
        throw new UnsupportedOperationException("not yet implemented");
    }

    @Override
    public SubmissionForPickup createSubmission(final SubmissionSubmit submission) {
        throw new UnsupportedOperationException("not yet implemented");
    }

    @Override
    public SubmissionForPickup sendSubmission(final UUID submissionId, final Metadata encryptedMetadata) {
        throw new UnsupportedOperationException("not yet implemented");
    }

    @Override
    public void uploadAttachments(final UUID submissionId, final List<Attachment> encryptedAttachments) {
        throw new UnsupportedOperationException("not yet implemented");
    }

    @Override
    public Attachment createAttachment(final File file) {
        try {
            final String hashedFileContent = hashUtil.createHashFromFile(file);
            final Hash__1 hash = new Hash__1();
            hash.setType(Type.SHA_512);
            hash.setContent(hashedFileContent);
            return Attachment.builder()
                    .attachmentId(UUID.randomUUID())
                    .filename(file.getName())
                    .hash(hash)
                    .build();
        } catch (final IOException e) {
            logger.error("Attachment could not be read", e);
            throw new AttachmentCreationException(e.getMessage(), e);
        }
    }

    @Override
    public RSAKey getEncryptionKeyForDestination(final UUID destinationId) {
        throw new UnsupportedOperationException("not yet implemented");
    }

}
